@@ -1,8 +1,8 @@
 # Adjust for exposure misclassification
 
 # the following functions feed into adjust_em():
-# adjust_em_val() (data_validation input),
-# adjust_em_coef() (bias_params input)
+# adjust_em_val() (data_validation input, method: imputation),
+# adjust_em_coef() (bias_params input, method: imputation)
 
 adjust_em_val <- function(
     data_observed,
@@ -132,114 +132,53 @@ adjust_em_coef <- function(
     y_binary <- FALSE
   }
 
+  # Extract X model coefficients
   x1_0 <- x_model_coefs[1]
   x1_xstar <- x_model_coefs[2]
   x1_y <- x_model_coefs[3]
+  x_coefs_c <- x_model_coefs[4:len_x_coefs]
 
-  if (is.null(confounders)) {
-    df <- data.frame(Xstar = xstar, Y = y)
-    df$Xpred <- rbinom(n, 1, plogis(x1_0 + x1_xstar * df$Xstar + x1_y * df$Y))
+  # Create base dataframe
+  df <- data.frame(Xstar = xstar, Y = y)
 
-    if (y_binary) {
-      final <- glm(
-        Y ~ Xpred,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ Xpred,
-        data = df
-      )
+  # Add confounders if they exist
+  if (!is.null(confounders)) {
+    for (i in seq_along(confounders)) {
+      df[[paste0("C", i)]] <- data[, confounders[i]]
     }
-  } else if (len_c == 1) {
-    c1 <- data[, confounders]
-    df <- data.frame(Xstar = xstar, Y = y, C1 = c1)
+  }
 
-    x1_c1 <- x_model_coefs[4]
+  # Construct X prediction formula dynamically
+  x_formula <- "x1_0 + x1_xstar * df$Xstar + x1_y * df$Y"
+  if (!is.null(confounders)) {
+    for (i in seq_along(confounders)) {
+      x_formula <- paste0(x_formula, " + x_coefs_c[", i, "] * df$C", i)
+    }
+  }
 
-    df$Xpred <- rbinom(
-      n, 1, plogis(
-        x1_0 + x1_xstar * df$Xstar +
-          x1_y * df$Y + x1_c1 * df$C1
-      )
+  # Calculate X predictions
+  df$Xpred <- rbinom(n, 1, plogis(eval(parse(text = x_formula))))
+
+  # Construct final model formula
+  model_terms <- c("Xpred")
+  if (!is.null(confounders)) {
+    model_terms <- c(model_terms, paste0("C", seq_along(confounders)))
+  }
+  model_formula <- as.formula(
+    paste("Y ~", paste(model_terms, collapse = " + "))
+  )
+
+  # Fit final model
+  if (y_binary) {
+    final <- glm(
+      model_formula,
+      family = binomial(link = "logit"),
+      data = df
     )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ Xpred + C1,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ Xpred + C1,
-        data = df
-      )
-    }
-  } else if (len_c == 2) {
-    c1 <- data[, confounders[1]]
-    c2 <- data[, confounders[2]]
-
-    df <- data.frame(Xstar = xstar, Y = y, C1 = c1, C2 = c2)
-
-    x1_c1 <- x_model_coefs[4]
-    x1_c2 <- x_model_coefs[5]
-
-    df$Xpred <- rbinom(
-      n, 1, plogis(
-        x1_0 + x1_xstar * df$Xstar + x1_y * df$Y +
-          x1_c1 * df$C1 + x1_c2 * df$C2
-      )
-    )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ Xpred + C1 + C2,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ Xpred + C1 + C2,
-        data = df
-      )
-    }
-  } else if (len_c == 3) {
-    c1 <- data[, confounders[1]]
-    c2 <- data[, confounders[2]]
-    c3 <- data[, confounders[3]]
-
-    df <- data.frame(Xstar = xstar, Y = y, C1 = c1, C2 = c2, C3 = c3)
-
-    x1_c1 <- x_model_coefs[4]
-    x1_c2 <- x_model_coefs[5]
-    x1_c3 <- x_model_coefs[6]
-
-    df$Xpred <- rbinom(
-      n, 1,
-      plogis(
-        x1_0 + x1_xstar * df$Xstar + x1_y * df$Y +
-          x1_c1 * df$C1 + x1_c2 * df$C2 + x1_c3 * df$C3
-      )
-    )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ Xpred + C1 + C2 + C3,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ Xpred + C1 + C2 + C3,
-        data = df
-      )
-    }
-  } else if (len_c > 3) {
-    stop(
-      "This function is currently not compatible with >3 confounders.",
-      call. = FALSE
+  } else {
+    final <- lm(
+      model_formula,
+      data = df
     )
   }
 
@@ -252,17 +191,7 @@ adjust_em <- function(
     data_validation = NULL,
     bias_params = NULL,
     level = 0.95) {
-  if (
-    (!is.null(data_validation) && !is.null(bias_params)) ||
-      (is.null(data_validation) && is.null(bias_params))
-  ) {
-    stop(
-      "One of data_validation or bias_params must be non-null.",
-      call. = FALSE
-    )
-  }
   data <- data_observed$data
-
   xstar <- data[, data_observed$exposure]
   y <- data[, data_observed$outcome]
 
@@ -290,23 +219,5 @@ adjust_em <- function(
     )
   }
 
-  est <- summary(final)$coef[2, 1]
-  se <- summary(final)$coef[2, 2]
-  alpha <- 1 - level
-
-  if (y_binary) {
-    estimate <- exp(est)
-    ci <- c(
-      exp(est + se * qnorm(alpha / 2)),
-      exp(est + se * qnorm(1 - alpha / 2))
-    )
-  } else {
-    estimate <- est
-    ci <- c(
-      est + se * qnorm(alpha / 2),
-      est + se * qnorm(1 - alpha / 2)
-    )
-  }
-
-  return(list(estimate = estimate, ci = ci))
+  calculate_results(final, level, y_binary)
 }
