@@ -1,8 +1,8 @@
 # Adjust for uncontrolled confounding
 
 # the following functions feed into adjust_uc():
-# adjust_uc_val() (data_validation input),
-# adjust_uc_coef() (bias_params input)
+# adjust_uc_val() (data_validation input, method: imputation),
+# adjust_uc_coef() (bias_params input, method: imputation)
 
 adjust_uc_val <- function(
     data_observed,
@@ -132,110 +132,53 @@ adjust_uc_coef <- function(
     y_binary <- FALSE
   }
 
+  # Extract U model coefficients
   u1_0 <- u_model_coefs[1]
   u1_x <- u_model_coefs[2]
   u1_y <- u_model_coefs[3]
+  u_coefs_c <- u_model_coefs[4:len_u_coefs]
 
-  if (is.null(confounders)) {
-    df <- data.frame(X = x, Y = y)
-    df$Upred <- rbinom(n, 1, plogis(u1_0 + u1_x * df$X + u1_y * df$Y))
+  # Create base dataframe
+  df <- data.frame(X = x, Y = y)
 
-    if (y_binary) {
-      final <- glm(
-        Y ~ X + Upred,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ X + Upred,
-        data = df
-      )
+  # Add confounders if they exist
+  if (!is.null(confounders)) {
+    for (i in seq_along(confounders)) {
+      df[[paste0("C", i)]] <- data[, confounders[i]]
     }
-  } else if (len_c == 1) {
-    c1 <- data[, confounders]
-    df <- data.frame(X = x, Y = y, C1 = c1)
+  }
 
-    u1_c1 <- u_model_coefs[4]
+  # Construct U prediction formula dynamically
+  u_formula <- "u1_0 + u1_x * df$X + u1_y * df$Y"
+  if (!is.null(confounders)) {
+    for (i in seq_along(confounders)) {
+      u_formula <- paste0(u_formula, " + u_coefs_c[", i, "] * df$C", i)
+    }
+  }
 
-    df$Upred <- rbinom(
-      n, 1, plogis(u1_0 + u1_x * df$X + u1_y * df$Y + u1_c1 * df$C1)
+  # Calculate U predictions
+  df$Upred <- rbinom(n, 1, plogis(eval(parse(text = u_formula))))
+
+  # Construct final model formula
+  model_terms <- c("X", "Upred")
+  if (!is.null(confounders)) {
+    model_terms <- c(model_terms, paste0("C", seq_along(confounders)))
+  }
+  model_formula <- as.formula(
+    paste("Y ~", paste(model_terms, collapse = " + "))
+  )
+
+  # Fit final model
+  if (y_binary) {
+    final <- glm(
+      model_formula,
+      family = binomial(link = "logit"),
+      data = df
     )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ X + C1 + Upred,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ X + C1 + Upred,
-        data = df
-      )
-    }
-  } else if (len_c == 2) {
-    c1 <- data[, confounders[1]]
-    c2 <- data[, confounders[2]]
-
-    df <- data.frame(X = x, Y = y, C1 = c1, C2 = c2)
-
-    u1_c1 <- u_model_coefs[4]
-    u1_c2 <- u_model_coefs[5]
-
-    df$Upred <- rbinom(
-      n, 1, plogis(
-        u1_0 + u1_x * df$X + u1_y * df$Y + u1_c1 * df$C1 + u1_c2 * df$C2
-      )
-    )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ X + C1 + C2 + Upred,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ X + C1 + C2 + Upred,
-        data = df
-      )
-    }
-  } else if (len_c == 3) {
-    c1 <- data[, confounders[1]]
-    c2 <- data[, confounders[2]]
-    c3 <- data[, confounders[3]]
-
-    df <- data.frame(X = x, Y = y, C1 = c1, C2 = c2, C3 = c3)
-
-    u1_c1 <- u_model_coefs[4]
-    u1_c2 <- u_model_coefs[5]
-    u1_c3 <- u_model_coefs[6]
-
-    df$Upred <- rbinom(
-      n, 1,
-      plogis(
-        u1_0 + u1_x * df$X + u1_y * df$Y +
-          u1_c1 * df$C1 + u1_c2 * df$C2 + u1_c3 * df$C3
-      )
-    )
-
-    if (y_binary) {
-      final <- glm(
-        Y ~ X + C1 + C2 + C3 + Upred,
-        family = binomial(link = "logit"),
-        data = df
-      )
-    } else {
-      final <- lm(
-        Y ~ X + C1 + C2 + C3 + Upred,
-        data = df
-      )
-    }
-  } else if (len_c > 3) {
-    stop(
-      "This function is currently not compatible with >3 confounders.",
-      call. = FALSE
+  } else {
+    final <- lm(
+      model_formula,
+      data = df
     )
   }
 
@@ -248,17 +191,7 @@ adjust_uc <- function(
     data_validation = NULL,
     bias_params = NULL,
     level = 0.95) {
-  if (
-    (!is.null(data_validation) && !is.null(bias_params)) ||
-      (is.null(data_validation) && is.null(bias_params))
-  ) {
-    stop(
-      "One of data_validation or bias_params must be non-null.",
-      call. = FALSE
-    )
-  }
   data <- data_observed$data
-
   x <- data[, data_observed$exposure]
   y <- data[, data_observed$outcome]
 
@@ -286,23 +219,5 @@ adjust_uc <- function(
     )
   }
 
-  est <- summary(final)$coef[2, 1]
-  se <- summary(final)$coef[2, 2]
-  alpha <- 1 - level
-
-  if (y_binary) {
-    estimate <- exp(est)
-    ci <- c(
-      exp(est + se * qnorm(alpha / 2)),
-      exp(est + se * qnorm(1 - alpha / 2))
-    )
-  } else {
-    estimate <- est
-    ci <- c(
-      est + se * qnorm(alpha / 2),
-      est + se * qnorm(1 - alpha / 2)
-    )
-  }
-
-  return(list(estimate = estimate, ci = ci))
+  calculate_results(final, level, y_binary)
 }
