@@ -1,10 +1,6 @@
 set.seed(1234)
-n <- 50000
 nreps <- 10
 
-# cont Y just for testing that function runs
-df_uc_sel$Y_cont <- plogis(df_uc_sel$Y) + rnorm(nrow(df_uc_sel), mean = 0, sd = 0.1)
-
 # 0 confounders
 
 nobias_model <- glm(
@@ -12,6 +8,7 @@ nobias_model <- glm(
   family = binomial(link = "logit"),
   data = df_uc_sel_source
 )
+or_true <- exp(summary(nobias_model)$coef[2, 1])
 
 u_model <- glm(
   U ~ X + Y,
@@ -43,29 +40,16 @@ single_run <- adjust_uc_sel(
   bias_params = bp_uc_sel
 )
 
-est <- vector()
-for (i in 1:nreps) {
-  bdf <- df_uc_sel[sample(seq_len(n), n, replace = TRUE), ]
-  df_observed <- data_observed(
-    bdf,
-    bias = c("uc", "sel"),
-    exposure = "X",
-    outcome = "Y",
-    confounders = NULL
-  )
-  results <- adjust_uc_sel(
-    df_observed,
-    bias_params = bp_uc_sel
-  )
-  est[i] <- results$estimate
-}
-
-or_true <- exp(summary(nobias_model)$coef[2, 1])
-or_adjusted <- median(est)
+bs_run <- multibias_adjust(
+  df_observed,
+  bias_params = bp_uc_sel,
+  bootstrap = TRUE,
+  bootstrap_reps = nreps
+)
 
 test_that("odds ratio and confidence interval output", {
-  expect_gt(or_adjusted, or_true - 0.1)
-  expect_lt(or_adjusted, or_true + 0.1)
+  expect_gt(bs_run$estimate, or_true - 0.1)
+  expect_lt(bs_run$estimate, or_true + 0.1)
   expect_vector(
     single_run$ci,
     ptype = double(),
@@ -80,6 +64,7 @@ nobias_model <- glm(
   family = binomial(link = "logit"),
   data = df_uc_sel_source
 )
+or_true <- exp(summary(nobias_model)$coef[2, 1])
 
 u_model <- glm(
   U ~ X + Y + C1 + C2 + C3,
@@ -111,29 +96,16 @@ single_run <- adjust_uc_sel(
   bias_params = bp_uc_sel
 )
 
-est <- vector()
-for (i in 1:nreps) {
-  bdf <- df_uc_sel[sample(seq_len(n), n, replace = TRUE), ]
-  df_observed <- data_observed(
-    bdf,
-    bias = c("uc", "sel"),
-    exposure = "X",
-    outcome = "Y",
-    confounders = c("C1", "C2", "C3")
-  )
-  results <- adjust_uc_sel(
-    df_observed,
-    bias_params = bp_uc_sel
-  )
-  est[i] <- results$estimate
-}
-
-or_true <- exp(summary(nobias_model)$coef[2, 1])
-or_adjusted <- median(est)
+bs_run <- multibias_adjust(
+  df_observed,
+  bias_params = bp_uc_sel,
+  bootstrap = TRUE,
+  bootstrap_reps = nreps
+)
 
 test_that("odds ratio and confidence interval output", {
-  expect_gt(or_adjusted, or_true - 0.1)
-  expect_lt(or_adjusted, or_true + 0.1)
+  expect_gt(bs_run$estimate, or_true - 0.1)
+  expect_lt(bs_run$estimate, or_true + 0.1)
   expect_vector(
     single_run$ci,
     ptype = double(),
